from bz2 import BZ2File
from pprint import pprint

from __init__ import Parser

bzfile_path = '/Users/gregor/Downloads/dewiktionary-latest-pages-articles-multistream.xml.bz2'
bz = BZ2File(bzfile_path)

for record in Parser(bz):
    pprint(record)



import re
from typing import Dict, Literal, Union

"""
Reference:
https://de.wiktionary.org/wiki/Kategorie:Wiktionary:Flexionstabelle_(Deutsch)
https://de.wiktionary.org/wiki/Hilfe:Flexionstabellen
"""
FlexionInfo = Dict[Literal['flexion'], Dict[str, str]]
FlexionResult = Union[Literal[False], FlexionInfo]

wanted_table_names = [
    'Deutsch Adjektiv Übersicht',
    'Deutsch Adverb Übersicht',
    'Deutsch Eigenname Übersicht',
    'Deutsch Nachname Übersicht',
    'Deutsch Pronomen Übersicht',
    'Deutsch Substantiv Übersicht',
    'Deutsch Substantiv Übersicht -sch',
    'Deutsch adjektivisch Übersicht',
    'Deutsch Toponym Übersicht',
    'Deutsch Verb Übersicht',
]


def find_table(text):
    re_string = '({{(' + '|'.join(wanted_table_names) + ')[^}]+}})'
    match_table = re.search(re_string, text)

    if not match_table:
        return False

    return match_table.group(1)


def parse_table_values(table_string):
    table_values = re.findall(r'(?:\|([^=\n]+)=([^\n|}]+))+?', table_string, re.MULTILINE)

    if not table_values:
        return False

    # normalize values
    result = {}
    for key, text in table_values:
        if key.startswith('Bild'):
            continue

        if key in ('Flexion', 'Weitere Konjugationen'):
            continue

        # clean text
        text = text.strip()
        text = text.replace('&nbsp', ' ')
        text = re.sub(r'<[^>]+>', ' ', text)  # strip comments, <ref>-tags etc.

        # genus
        if key in ['Genus', 'Genus 1', 'Genus 2', 'Genus 3', 'Genus 4']:
            # the Genus of plural words is set to 0 (or other value)
            # reference: https://de.wiktionary.org/wiki/Wiktionary:Teestube/Archiv/2015/11#Genus_in_der_Flexionstabelle_bei_Pluralw%C3%B6rtern
            # -> normalize
            text = text.lower()
            if text not in ['f', 'm', 'n']:
                continue

        if not text or text in ('—', '-', '–', '−', '?'):
            continue

        result[key] = text

    return result if result.keys() else False


def init(
    title: str,
    text: str,
    current_record
) -> FlexionResult:
    table_string = find_table(text)
    if not table_string:
        return False

    table_dict = parse_table_values(table_string)
    if table_dict is False:
        return False

    return {'flexion': table_dict}



import re
from typing import Dict, List, Literal, Union
import pyphen

SyllablesInfo = Dict[Literal['syllables'], List[str]]
SyllablesResult = Union[Literal[False], SyllablesInfo]

"""
Test strings:

{{Worttrennung}}
:ge·sagt, ge·tan

{{Worttrennung}}
:Ton {{Pl.}} To·ne

{{Worttrennung}}
:neun·zehn

{{Worttrennung}}
:bald, {{Komp.}} eher, {{Sup.}} am ehes·ten

{{Worttrennung}}
:ge·sagt, ge·tan (umgangssprachlich oft: gra·de, grad)

{{Worttrennung}}
:zwan·zig, zwan·zi·ge

{{Worttrennung}}
:re·den, {{Prät.}} re·de·te, {{Part.}} ge·re·det

{{Worttrennung}}
:re·den, {{Pl.}} re·de·nen

{{Worttrennung}}
:drei·ßig, ''[[veraltend]]:'' drei·ßi·ge

{{Worttrennung}}
:ein·und·zwan·zig

{{Worttrennung}}
:pa·cken, {{Prät.}} pack·te, {{Part.}} ge·packt

{{Worttrennung}}
:Skiz·ze, {{Pl.}} Skiz·zen

{{Worttrennung}}
:Me·lo·ne, {{Pl.}} Me·lo·nen

{{Worttrennung}}
:Narr, {{Pl.}} Nar·ren

{{Worttrennung}}
:{{kSg.}}, Ver·ein·te Na·ti·o·nen

{{Worttrennung}}
:Pa·cker, {{Pl.}} Pa·cker

{{Worttrennung}}
:''Neue Worttrennung:'' Mü·cken·stich, {{Pl.}} Mü·cken·sti·che
:''Alte Worttrennung:'' Mük·ken·stich, {{Pl.}} Mük·ken·sti·che

{{Worttrennung}}
:Lie·bes·kum·mer, {{kPl.}}

{{Worttrennung}}
:Pa·cke·rin, {{Pl.}} Pa·cke·rin·nen

{{Worttrennung}}
:rei·zen, {{Prät.}} reiz·te, {{Part.}} ge·reizt

{{Worttrennung}}
:rei·zen

{{Worttrennung}}
:pa·la·vern, {{Prät.}} pa·la·ver·te, {{Part.}} pa·la·vert

{{Worttrennung}}
:Toll·patsch, {{Pl.}} Toll·pat·sche

{{Worttrennung}}
:kir·ren, {{Prät.}} kirr·te, {{Part.}} ge·kirrt

{{Worttrennung}}
:Arzt, {{Pl.}} Ärz·te

{{Worttrennung}}
:Arzt, {{Pl.}} Arzts

{{Worttrennung}}
:Pflan·ze, {{Pl.}} Pflan·zen

{{Worttrennung}}
:, {{Pl.}}

{{Worttrennung}}
:
:ein·wir·ken, {{Prät.}} wirk·te ein, {{Part.}} ein·ge·wirkt

"""


def init(
    title: str,
    text: str,
    current_record
) -> SyllablesResult:
    # find syllables in wikitext
    # reference: https://de.wiktionary.org/wiki/Hilfe:Worttrennung
    result = False

    # strip comments, <ref>-tags etc.
    text = re.sub(r'<[^>]+>', ' ', text)

    # find paragraph
    match_p = re.search(r'{{Worttrennung}}\n(?::?\n)?([^\n]+)', text)
    if match_p is not None:
        paragraph = match_p.group(1)

        """
        Find title in wikitext under "{{Worttrennung}}".
        Keep characters, that are also part of title.
        Find start and end index.
        Paragraph can have commas/semicolons, but we don't know if they are part of title.
        valid:
            ge·sagt, ge·tan
        invalid (inflected form):
            zwan·zig, zwan·zi·ge
            In·tel·li·genz·quo·ti·ent; In·tel·li·genz·quo·ti·en·ten

        """
        title_index = 0
        start_index = -1
        end_index = -1
        last_title_index = len(title) - 1
        last_paragraph_index = len(paragraph) - 1
        for index, char in enumerate(paragraph):
            # get start index
            # test if title can be inserted from current index
            # remove mid dots for testing
            if start_index == -1:
                if paragraph[index:].replace('·', '').startswith(title):
                    start_index = index
                    end_index = index
                continue

            end_index += 1
            if char == '·':
                continue

            title_index += 1
            if title_index >= last_title_index or char != title[title_index] or index == last_paragraph_index:
                end_index += 1
                break

        # remove everything after actual_index
        clean_string = paragraph[start_index:end_index]

        # split syllables, remove empty strings (ugly side effect of re.split)
        result = list(filter(None, re.split(r' |·|-', clean_string)))

    if not result and 'lang_code' in current_record and current_record['lang_code'] in pyphen.LANGUAGES:
        # get syllables with PyHyphen
        dic = pyphen.Pyphen(lang=current_record['lang_code'])
        syl_string = dic.inserted(title)
        # split by "-" and remove empty entries
        result = [x for x in re.split(r' |-', syl_string) if x]

    return {'syllables': result} if result else False



import re
from pathlib import Path
from importlib.machinery import SourceFileLoader
from typing import Any, Callable, Iterator, List, Tuple, TypedDict

from lxml import etree

from wiktionary_de_parser.methods.flexion import FlexionInfo
from wiktionary_de_parser.methods.ipa import IPAInfo
from wiktionary_de_parser.methods.pos import POSInfo
from wiktionary_de_parser.methods.syllables import SyllablesInfo

PACKAGE_PATH = Path(__file__).parent.absolute()


class _Record(TypedDict):
    # following this cumbersome example to make some fields required:
    # https://www.python.org/dev/peps/pep-0655/#motivation
    title: str
    wikitext: str
    lemma: str
    inflected: bool


class Record(_Record, total=False):
    flexion: FlexionInfo
    ipa: IPAInfo
    lang: str
    lang_code: str
    pos: POSInfo
    syllables: SyllablesInfo


class Parser:
    def __init__(
        self,
        source: Any,
        custom_methods: List[Callable] = [],
        ignored_prefixes: Tuple[str, ...] = ('mediawiki:', 'vorlage:', 'wiktionary:', 'hilfe:', 'flexion:',
                                             'datei:', 'verzeichnis:', 'kategorie:', 'reim:', 'modul:', 'fn:')
    ) -> None:
        # Initialize 'iterparse' to only generate 'end' events on tag '<entity>'
        # Credits: https://stackoverflow.com/a/55147982/5732518
        # Prepend the default Namespace {*} to get anything.
        self.context = etree.iterparse(source, events=("end",), tag=['{*}' + 'page'])

        # ignore page titles starting with these prefixes followed by ":"
        self.ignored_prefixes = ignored_prefixes

        # load default & custom methods
        self.extraction_methods: List[Callable] = []
        self.load_methods(custom_methods)

    def load_methods(self, custom_methods: List[Callable]) -> None:
        # load extraction methods from folder
        methods_path = PACKAGE_PATH.joinpath('methods')
        method_files = [f for f in methods_path.iterdir() if not f.name.startswith('__') and f.name.endswith('.py')]

        for idx, file_path in enumerate(method_files):
            fullname = 'method-' + str(idx)
            module: Any = SourceFileLoader(fullname=fullname, path=str(file_path)).load_module()

            if not hasattr(module, 'init'):
                raise Exception(f'No init() method found in file "{file_path.name}"')
            if not callable(module.init):
                raise Exception(f'init() method in "{file_path.name}" is not callable')

            # append method
            self.extraction_methods.append(module.init)

        # load custom exctraction methods
        if custom_methods and isinstance(custom_methods, list):
            for method in custom_methods:
                if not callable(method):
                    raise Exception(f'Provided extraction method "{str(method)}" is not callable')
                self.extraction_methods.append(method)

    def parse_page(self) -> Iterator[Tuple[str, str]]:
        """
        Parse the XML File for title and revision tag
        Clear/Delete the Element Tree after processing

        :return: yield title and revision wikitext
        """
        for event, elem in self.context:
            # Assign the 'elem.namespace' to the 'xpath'
            elem_namespace: str = etree.QName(elem).namespace

            _title: List[str] = elem.xpath('./xmlns:title/text( )', namespaces={'xmlns': elem_namespace})
            _wikitext: List[str] = elem.xpath('./xmlns:revision/xmlns:text/text( )',
                                              namespaces={'xmlns': elem_namespace})

            title = _title[0] if _title else ''
            wikitext = _wikitext[0] if _wikitext else ''

            yield title, wikitext

            elem.clear()
            while elem.getprevious() is not None:
                del elem.getparent()[0]

    def parse_sections(self, wikitext: str) -> List[str]:
        """
        Split page into sections. One page can have multiple word sections, for example:
            - https://de.wiktionary.org/wiki/instrument

        New sections begin at "==" and "===" (sometimes there is no "==")
        Compare:
            - https://de.wiktionary.org/wiki/instrument
            - https://de.wiktionary.org/wiki/Becken
        """
        sections: List[str] = re.findall(r'(=== {{Wortart(?:[\w\W](?!^===? ))+)', wikitext, re.MULTILINE)

        for entry in sections:
            yield entry

    def __iter__(self) -> Iterator[Record]:
        """
        Iterate all pages yielded from self.parse_page() and all word sections yielded from self.parse_sections()

        :return: Dict with final result
        """
        for title, wikitext in self.parse_page():
            # check for ignored titles
            if title.lower().startswith(self.ignored_prefixes):
                continue

            for section_text in self.parse_sections(wikitext):
                current_record: Record = {
                    'title': title,
                    'wikitext': section_text,
                    'lemma': '', # to be overwritten
                    'inflected': False # might be overwritten
                }

                # execute parse methods & update current_record
                for method in self.extraction_methods:
                    data = method(title, section_text, current_record)
                    if data is False:
                        continue
                    current_record.update(data)

                yield current_record



import re
import itertools
from typing import Dict, List, Literal, Union

"""
reference: https://de.wiktionary.org/wiki/Hilfe:Wortart
"""
POSInfo = Dict[Literal['pos'], Dict[str, List[str]]]
POSResult = Union[Literal[False], POSInfo]
debug = False

pos_map = {
    'Abkürzung': [
        'Kurzwort'
    ],
    'Adjektiv': [
        'Partizip',
        'Partizip I',
        'Partizip II',
        'Komparativ',
        'Superlativ',
        'Gerundivum',
        'Dekliniertes Gerundivum',
    ],
    'Adposition': [
        'Postposition',
        'Präposition',
        'Zirkumposition',
    ],
    'Adverb': [
        'Fokuspartikel',
        'Gradpartikel',
        'Interrogativadverb',
        'Konjunktionaladverb',
        'Lokaladverb',
        'Modalpartikel',
        'Negationspartikel',
        'Pronominaladverb',
        'Temporaladverb',
        'Modaladverb',
        'Relativadverb',
    ],
    'Affix': [
        'Präfix',
        'Suffix',
        'Infix',
        'Interfix',
        'Zirkumfix',
        'Präfixoid',
        'Suffixoid',
    ],
    'Gebundenes Lexem': [],
    'Artikel': [],
    'Konjunktion': [
        'Subjunktion'
    ],
    'Kontraktion': [],
    'Numerale': [
        'Kardinalzahl',
        'Ordinalzahl',
    ],
    'Partikel': [
        'Interjektion',
        'Antwortpartikel',
        'Grußformel',
        'Onomatopoetikum',
        'Vergleichspartikel',
        'Fragepartikel',
    ],
    'Pronomen': [
        'Indefinitpronomen',
        'Interrogativpronomen',
        'Demonstrativpronomen',
        'Personalpronomen',
        'Possessivpronomen',
        'Reflexivpronomen',
        'Reflexives Personalpronomen',
        'Relativpronomen',
        'Reziprokpronomen',
    ],
    'Redewendung': [],
    'Sprichwort': [],
    'Geflügeltes Wort': [],
    'Merkspruch': [],
    'Formel': [],
    'Substantiv': [
        'Toponym',
        'Vorname',
        'Nachname',
        'Familienname',
        'Patronym',
        'Eigenname',
        'Straßenname',
        'Zahlklassifikator',
        'Singularetantum',
        'Pluraletantum',
        'adjektivische Deklination',
        'Substantivierter Infinitiv',
    ],
    'Ortsnamengrundwort': [],
    'Symbol': [
        'Buchstabe',
        'Zahlzeichen',
        'Schriftzeichen',
    ],
    'Verb': [
        'Konjugierte Form',
        'Hilfsverb',
        'Erweiterter Infinitiv',
    ],
    'Wortverbindung': [],

    # needs additional parsing; can be: Substantiv, Adjektiv, Artikel, Pronomen
    'Deklinierte Form': []
}

if debug is True:
    not_in_map = set()
    all_pos_names = list(pos_map.keys()) + list(itertools.chain.from_iterable(list(pos_map.values())))
    all_pos_names = [x.lower() for x in all_pos_names]


def find_pos(title, pos_names, text, current_record):
    result = {}

    # fix POS when there is a certain POS template, but POS is not in pos_names
    # example "Substantiv": https://de.wiktionary.org/wiki/wei%C3%9Fes_Gold

    # Substantiv
    if '{{Deutsch adjektivisch Übersicht' in text \
            or '{{Deutsch Substantiv Übersicht - sch' in text \
            or '{{Deutsch Substantiv Übersicht' in text \
            or '{{Deutsch Toponym Übersicht' in text:
        if 'Substantiv' not in result:
            result['Substantiv'] = []
    if '{{Deutsch adjektivisch Übersicht' in text and 'adjektivische Deklination' not in result['Substantiv']:
        result['Substantiv'].append('adjektivische Deklination')
    if '{{Deutsch Toponym Übersicht' in text and 'Toponym' not in result['Substantiv']:
        result['Substantiv'].append('Toponym')
    # Adjektiv
    if '{{Deutsch Adjektiv Übersicht' in text and 'Adjektiv' not in result:
        result['Adjektiv'] = []
    # Adverb
    if '{{Deutsch Adverb Übersicht' in text and 'Adverb' not in result:
        result['Adverb'] = []
    # Pronomen
    if '{{Deutsch Pronomen Übersicht' in text and 'Pronomen' not in result:
        result['Pronomen'] = []
    # Verb
    if '{{Deutsch Verb Übersicht' in text and 'Verb' not in result:
        result['Verb'] = []

    # map other pos names
    for key, values in pos_map.items():
        # maintain letter case of position map
        key_low = key.lower()
        for name in pos_names:
            name_low = name.lower()
            if name_low == key_low and key not in result:
                result[key] = []
            values_low = [x.lower() for x in values]
            if name_low in values_low:
                if key not in result:
                    result[key] = []
                name_idx = values_low.index(name_low)
                value = values[name_idx]
                if value not in result[key]:
                    result[key].append(value)

    if debug is True:
        not_found_names = [x for x in pos_names if x.lower() not in all_pos_names and x not in not_in_map]
        if not_found_names:
            not_in_map.update(not_found_names)
            for name in not_found_names:
                print('"{}" not in POS-map (all: {})'.format(name, ', '.join(pos_names)))

    return result


def init(
    title: str,
    text: str,
    current_record
) -> POSResult:
    # find line
    match_line = re.search(r'(=== ?{{Wortart(?:-Test)?\|[^\n]+)', text)
    if not match_line:
        return False

    # can have multiple POS values
    line = match_line.group(1)
    pos_names = re.findall(r'{{Wortart(?:-Test)?\|([^}|]+)(?:\|[^}|]+)*}}', line)
    if not pos_names:
        return False

    # strip
    pos_names = [name.strip() for name in pos_names]

    # find in map
    pos_normalized = find_pos(title, pos_names, text, current_record)
    if not pos_normalized.keys():
        return False

    return {'pos': pos_normalized}



import re
from typing import Literal, Optional, TypedDict, Union


class LemmaInfo(TypedDict):
    lemma: str
    inflected: bool


LemmaResult = LemmaInfo


def init(
    title: str,
    text: str,
    current_record
) -> LemmaResult:
    """
    Grundformverweis
    Von einer Deklination spricht man beim Beugen von Substantiven und den
    dazugehörigen Adjektiven und Pronomen (Nominalflexion),
    von einer Konjugation beim Beugen eines Verbs (Verbalflexion).

    Reference:
    https://de.wiktionary.org/wiki/Kategorie:Flektierte_Form_(Deutsch)
    https://de.wiktionary.org/wiki/Vorlage:Grundformverweis_Konj
    https://de.wiktionary.org/wiki/Vorlage:Grundformverweis_Dekl
    """

    # match_test = re.search(r'({{Grundformverweis[^}]+}})', text)
    # if match_test:
    #     print(match_test.group(1))
    #     print()

    found_lemma = title
    inflected = False
    match_lemma = re.search(r'{{Grundformverweis[^|]*\|(?:\w+=[^\|]+\|)*([^\|\#\}]+)', text)
    if match_lemma:
        found_lemma = match_lemma.group(1).strip()
        inflected = True

    # title is lemma
    return {
        'lemma': found_lemma,
        'inflected': inflected
    }



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wiktionary_de_parser', 'wiktionary_de_parser.methods']

package_data = \
{'': ['*'], 'wiktionary_de_parser': ['assets/*']}

install_requires = \
['lxml>=4.6.4,<5.0.0', 'pyphen>=0.11.0,<0.12.0']

setup_kwargs = {
    'name': 'wiktionary-de-parser',
    'version': '0.8.8',
    'description': 'Extracts data from German Wiktionary dump files. Allows you to add your own extraction methods 🚀',
    'long_description': '# wiktionary_de_parser\n\n`wiktionary_de_parser` is a Python module to extract data from German Wiktionary XML files (for Python 3.7+). It allows you to add your own extraction methods.\n\n## Installation\n\n`pip3 install wiktionary_de_parser`\n\n## Features\n\n- comes with preset extraction methods for:\n  - flexion tables, genus, IPA, language, lemma, part of speech (basic), syllables, raw Wikitext\n- allows you to add your own extraction methods (pass them as argument)\n- data values are normalized and cleaned from obsolete Wikitext markup\n- yields per section, not per page (a word can have multiple meanings, which is why some Wiktionary pages have multiple \'sections\')\n\n## Usage\n\nImport `wiktionary_de_parser` like this:\n\n```python\nfrom bz2 import BZ2File\nfrom wiktionary_de_parser import Parser\n\nbzfile_path = \'/tmp/dewiktionary-latest-pages-articles-multistream.xml.bz2\'\nbz = BZ2File(bzfile_path)\n\nfor record in Parser(bz):\n    if \'lang_code\' not in record or record[\'lang_code\'] != \'de\':\n      continue\n    # do stuff with \'record\'\n```\n\nNote: in this example we use [BZ2File](https://pypi.org/project/bz2file/) to read a compressed Wiktionary dump file.\nThe Wiktionary dump file is obtained from [here](https://dumps.wikimedia.org/dewiktionary/).\n\n### Adding new extraction methods\n\nAll extraction methods must return a `Dict()` and accept the following arguments:\n\n- `title` (_string_): The title of the current Wiktionary page\n- `text` (_string_): The [Wikitext](https://en.wikipedia.org/wiki/Wiki#Editing) of the current word entry/section\n- `current_record` (_Dict_): A dictionary with all values of the current iteration (e. g. `current_record[\'lang_code\']`)\n\n```python\n# Create a new extraction method\ndef my_method(title, text, current_record):\n  # do stuff\n  return {\'my_field\': my_data}\n\n# Pass a list with all extraction methods to the class constructor:\nfor record in Parser(bz, custom_methods=[my_method]):\n    print(record[\'my_field\'])\n```\n\n## Sample data:\n\n```python\n{\'flexion\': {\'Akkusativ Plural\': \'Trittbrettfahrer\',\n             \'Akkusativ Singular\': \'Trittbrettfahrer\',\n             \'Dativ Plural\': \'Trittbrettfahrern\',\n             \'Dativ Singular\': \'Trittbrettfahrer\',\n             \'Genitiv Plural\': \'Trittbrettfahrer\',\n             \'Genitiv Singular\': \'Trittbrettfahrers\',\n             \'Genus\': \'m\',\n             \'Nominativ Plural\': \'Trittbrettfahrer\',\n             \'Nominativ Singular\': \'Trittbrettfahrer\'},\n \'inflected\': False,\n \'ipa\': [\'ˈtʁɪtbʁɛtˌfaːʁɐ\'],\n \'lang\': \'Deutsch\',\n \'lang_code\': \'de\',\n \'lemma\': \'Trittbrettfahrer\',\n \'pos\': {\'Substantiv\': []},\n \'syllables\': [\'Tritt\', \'brett\', \'fah\', \'rer\'],\n \'title\': \'Trittbrettfahrer\',\n \'wikitext\': \'=== {{Wortart|Substantiv|Deutsch}}, {{m}} ===\\n\'\n             \'\\n\'\n             \'{{Deutsch Substantiv Übersicht\\n\'\n             \'|Genus=m\\n\'\n             \'|Nominativ Singular=Trittbrettfahrer\\n\'\n             \'|Nominativ Plural=Trittbrettfahrer\\n\'\n             \'|Genitiv Singular=Trittbrettfahrers\\n\'\n             \'|Genitiv Plural=Trittbrettfahrer\\n\'\n             \'|Dativ Singular=Trittbrettfahrer\\n\'\n             \'|Dativ Plural=Trittbrettfahrern\\n\'\n             \'|Akkusativ Singular=Trittbrettfahrer\\n\'\n             \'|Akkusativ Plural=Trittbrettfahrer\\n\'\n             \'}}\\n\'\n             \'\\n\'\n             \'{{Worttrennung}}\\n\'\n             \':Tritt·brett·fah·rer, {{Pl.}} Tritt·brett·fah·rer\\n\'\n             \'\\n\'\n             \'{{Aussprache}}\\n\'\n             \':{{IPA}} {{Lautschrift|ˈtʁɪtbʁɛtˌfaːʁɐ}}\\n\'\n             \':{{Hörbeispiele}} {{Audio|}}\\n\'\n             \'\\n\'\n             \'{{Bedeutungen}}\\n\'\n             \':[1] Person, die ohne [[Anstrengung]] an Vorteilen teilhaben \'\n             \'will\\n\'\n             \'\\n\'\n             \'{{Herkunft}}\\n\'\n             \':[[Determinativkompositum]] aus den Substantiven \'\n             "\'\'[[Trittbrett]]\'\' und \'\'[[Fahrer]]\'\'\\n"\n             \'\\n\'\n             \'{{Weibliche Wortformen}}\\n\'\n             \':[1] [[Trittbrettfahrerin]]\\n\'\n             \'\\n\'\n             \'{{Beispiele}}\\n\'\n             \':[1] „Bleibt schließlich noch das Problem der \'\n             "\'\'Trittbrettfahrer,\'\' die sich ohne Versicherung aus "\n             \'Nachlässigkeit in das soziale Netz abgleiten \'\n             \'lassen.“<ref>{{Internetquelle|url=http://books.google.se/books?id=VjLq84xNpfMC&pg=PA446&dq=trittbrettfahrer&hl=de&sa=X&ei=8AztU4aVJYq_ygOd1oKIDA&ved=0CEEQ6AEwBjgK#v=onepage&q=trittbrettfahrer&f=false|titel=Öffentliche \'\n             \'Finanzen in der Demokratie: Eine Einführung, Charles B. \'\n             \'Blankart|zugriff=2014-08-14}}</ref>\\n\'\n             \'\\n\'\n             \'{{Wortbildungen}}\\n\'\n             \':[1] [[Trittbrettfahrer-Problem]]\\n\'\n             \'\\n\'\n             \'==== {{Übersetzungen}} ====\\n\'\n             \'{{Ü-Tabelle|Ü-links=\\n\'\n             \'*{{en}}: [1] {{Ü|en|free rider}}\\n\'\n             \'*{{fi}}: [1] {{Ü|fi|siipeilijä}}, {{Ü|fi|vapaamatkustaja}}\\n\'\n             \'*{{fr}}: [1] {{Ü|fr|profiteur}}\\n\'\n             \'|Ü-rechts=\\n\'\n             \'*{{it}}: [1] {{Ü|it|scroccone}} {{m}}\\n\'\n             \'*{{es}}: [1] {{Ü|es|}}\\n\'\n             \'}}\\n\'\n             \'\\n\'\n             \'{{Referenzen}}\\n\'\n             \':[1] {{Wikipedia|Trittbrettfahrer}}\\n\'\n             \':[*] {{Ref-DWDS|Trittbrettfahrer}}\\n\'\n             \':[*] {{Ref-Canoo|Trittbrettfahrer}}\\n\'\n             \':[1] {{Ref-UniLeipzig|Trittbrettfahrer}}\\n\'\n             \':[1] {{Ref-FreeDictionary|Trittbrettfahrer}}\\n\'\n             \'\\n\'\n             \'{{Quellen}}\'}\n```\n\n## License\n\n[MIT](https://github.com/gambolputty/wiktionary_de_parser/blob/master/LICENSE.md) © Gregor Weichbrodt\n',
    'author': 'Gregor Weichbrodt',
    'author_email': 'gregorweichbrodt@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/gambolputty/wiktionary_de_parser',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import re
from typing import Literal, TypedDict, Union

from wiktionary_de_parser import PACKAGE_PATH


class LangInfo(TypedDict, total=False):
    lang: str
    lang_code: str


LanguageResult = Union[Literal[False], LangInfo]

# https://de.wiktionary.org/wiki/Hilfe:Sprachcodes
langcodes = {}
with open(PACKAGE_PATH.joinpath('assets/sprachcodes_iso639-1.txt'), encoding='utf-8') as f:
    lines = f.read().split('\n')
    for line in lines:
        x = line.split(',')
        langcodes[x[0]] = x[1]


def init(
    title: str,
    text: str,
    current_record
) -> LanguageResult:
    match_lang_name = re.search(r'=== ?{{Wortart(?:-Test)?\|[^}|]+\|([^}|]+)(?:\|[^}|]+)*}}', text)
    if not match_lang_name:
        return False

    lang_name: str = match_lang_name.group(1) if match_lang_name.group(1) else match_lang_name.group(2)
    lang_name = lang_name.strip()

    # get lang code
    result: LangInfo = {'lang': lang_name}

    lang_name_low = lang_name.lower()
    if lang_name_low in langcodes:
        result['lang_code'] = langcodes[lang_name_low]

    return result
